@@ -46,8 +46,6 @@ class DHTConnection;
 class DHTMessageFactory;
 class DHTRoutingTable;
 
-class Logger;
-
 class DHTMessageReceiver {
 private:
   SharedHandle<DHTMessageTracker> tracker_;
@@ -58,8 +56,6 @@ private:
 
   SharedHandle<DHTRoutingTable> routingTable_;
 
-  Logger* logger_;
-
   SharedHandle<DHTMessage>
   handleUnknownMessage(const unsigned char* data, size_t length,
                        const std::string& remoteAddr, uint16_t remotePort);
