@@ -48,14 +48,13 @@ class DHTNode;
 class DHTBucket;
 class DHTTaskQueue;
 class DHTTaskFactory;
-class BNode;
-class Logger;
+class DHTBucketTreeNode;
 
 class DHTRoutingTable {
 private:
   SharedHandle<DHTNode> localNode_;
 
-  BNode* root_;
+  DHTBucketTreeNode* root_;
 
   size_t numBucket_;
 
@@ -63,8 +62,6 @@ private:
 
   SharedHandle<DHTTaskFactory> taskFactory_;
 
-  Logger* logger_;
-
   bool addNode(const SharedHandle<DHTNode>& node, bool good);
 public:
   DHTRoutingTable(const SharedHandle<DHTNode>& localNode);
