@@ -55,19 +55,20 @@
 #include "RequestGroupMan.h"
 #include "RequestGroup.h"
 #include "util.h"
+#include "fmt.h"
 
 namespace aria2 {
 
-DefaultBtMessageDispatcher::DefaultBtMessageDispatcher():
-  cuid(0),
-  requestTimeout_(0),
-  logger_(LogFactory::getInstance()) {}
+DefaultBtMessageDispatcher::DefaultBtMessageDispatcher()
+  : cuid_(0),
+    messageFactory_(0),
+    requestGroupMan_(0),
+    requestTimeout_(0)
+{}
 
 DefaultBtMessageDispatcher::~DefaultBtMessageDispatcher()
 {
-  if(logger_->debug()) {
-    logger_->debug("DefaultBtMessageDispatcher::deleted");
-  }
+  A2_LOG_DEBUG("DefaultBtMessageDispatcher::deleted");
 }
 
 void DefaultBtMessageDispatcher::addMessageToQueue
@@ -145,26 +146,23 @@ class AbortOutstandingRequest {
 private:
   SharedHandle<Piece> piece_;
   cuid_t cuid_;
-  Logger* logger_;
 public:
-  AbortOutstandingRequest(const SharedHandle<Piece>& piece, cuid_t cuid):
-    piece_(piece),
-    cuid_(cuid),
-    logger_(LogFactory::getInstance()) {}
+  AbortOutstandingRequest(const SharedHandle<Piece>& piece, cuid_t cuid)
+    : piece_(piece),
+      cuid_(cuid)
+  {}
 
   void operator()(const RequestSlot& slot) const
   {
-    if(logger_->debug()) {
-      logger_->debug(MSG_DELETING_REQUEST_SLOT,
-                     util::itos(cuid_).c_str(),
-                     slot.getIndex(),
-                     slot.getBlockIndex());
-      logger_->debug("index=%d, begin=%d", slot.getIndex(), slot.getBegin());
-    }
+    A2_LOG_DEBUG(fmt(MSG_DELETING_REQUEST_SLOT,
+                     cuid_,
+                     static_cast<unsigned long>(slot.getIndex()),
+                     slot.getBegin(),
+                     static_cast<unsigned long>(slot.getBlockIndex())));
     piece_->cancelBlock(slot.getBlockIndex());
   }
 };
-}
+} // namespace
 
 // localhost cancels outstanding download requests to the peer.
 void DefaultBtMessageDispatcher::doAbortOutstandingRequestAction
@@ -177,7 +175,7 @@ void DefaultBtMessageDispatcher::doAbortOutstandingRequestAction
   std::deque<RequestSlot>::iterator last =
     std::lower_bound(requestSlots_.begin(), requestSlots_.end(), rs);
 
-  std::for_each(first, last, AbortOutstandingRequest(piece, cuid));
+  std::for_each(first, last, AbortOutstandingRequest(piece, cuid_));
   requestSlots_.erase(first, last);
 
   BtAbortOutstandingRequestEvent event(piece);
@@ -194,33 +192,31 @@ private:
   cuid_t cuid_;
   SharedHandle<Peer> peer_;
   SharedHandle<PieceStorage> pieceStorage_;
-  Logger* logger_;
 public:
-  ProcessChokedRequestSlot(cuid_t cuid,
-                           const SharedHandle<Peer>& peer,
-                           const SharedHandle<PieceStorage>& pieceStorage):
-    cuid_(cuid),
-    peer_(peer),
-    pieceStorage_(pieceStorage),
-    logger_(LogFactory::getInstance()) {}
+  ProcessChokedRequestSlot
+  (cuid_t cuid,
+   const SharedHandle<Peer>& peer,
+   const SharedHandle<PieceStorage>& pieceStorage)
+    : cuid_(cuid),
+      peer_(peer),
+      pieceStorage_(pieceStorage)
+  {}
   
   void operator()(const RequestSlot& slot) const
   {
     if(!peer_->isInPeerAllowedIndexSet(slot.getIndex())) {
-      if(logger_->debug()) {
-        logger_->debug(MSG_DELETING_REQUEST_SLOT_CHOKED,
-                       util::itos(cuid_).c_str(),
-                       slot.getIndex(),
-                       slot.getBlockIndex());
-        logger_->debug("index=%d, begin=%d", slot.getIndex(), slot.getBegin());
-      }
+      A2_LOG_DEBUG(fmt(MSG_DELETING_REQUEST_SLOT_CHOKED,
+                       cuid_,
+                       static_cast<unsigned long>(slot.getIndex()),
+                       slot.getBegin(),
+                       static_cast<unsigned long>(slot.getBlockIndex())));
       SharedHandle<Piece> piece = pieceStorage_->getPiece(slot.getIndex());
       piece->cancelBlock(slot.getBlockIndex());
     }
   }
 
 };
-}
+} // namespace
 
 namespace {
 class FindChokedRequestSlot {
@@ -235,13 +231,13 @@ public:
     return !peer_->isInPeerAllowedIndexSet(slot.getIndex());
   }
 };
-}
+} // namespace
 
 // localhost received choke message from the peer.
 void DefaultBtMessageDispatcher::doChokedAction()
 {
   std::for_each(requestSlots_.begin(), requestSlots_.end(),
-                ProcessChokedRequestSlot(cuid, peer_, pieceStorage_));
+                ProcessChokedRequestSlot(cuid_, peer_, pieceStorage_));
 
   requestSlots_.erase(std::remove_if(requestSlots_.begin(), requestSlots_.end(),
                                      FindChokedRequestSlot(peer_)),
@@ -266,41 +262,39 @@ private:
   SharedHandle<Peer> peer_;
   SharedHandle<PieceStorage> pieceStorage_;
   BtMessageDispatcher* messageDispatcher_;
-  WeakHandle<BtMessageFactory> messageFactory_;
+  BtMessageFactory* messageFactory_;
   time_t requestTimeout_;
-  Logger* logger_;
 public:
-  ProcessStaleRequestSlot(cuid_t cuid, const SharedHandle<Peer>& peer,
-                          const SharedHandle<PieceStorage>& pieceStorage,
-                          BtMessageDispatcher* dispatcher,
-                          const WeakHandle<BtMessageFactory>& factory,
-                          time_t requestTimeout):
-    cuid_(cuid),
-    peer_(peer),
-    pieceStorage_(pieceStorage),
-    messageDispatcher_(dispatcher),
-    messageFactory_(factory),
-    requestTimeout_(requestTimeout),
-    logger_(LogFactory::getInstance()) {}
+  ProcessStaleRequestSlot
+  (cuid_t cuid, const SharedHandle<Peer>& peer,
+   const SharedHandle<PieceStorage>& pieceStorage,
+   BtMessageDispatcher* dispatcher,
+   BtMessageFactory* factory,
+   time_t requestTimeout)
+    : cuid_(cuid),
+      peer_(peer),
+      pieceStorage_(pieceStorage),
+      messageDispatcher_(dispatcher),
+      messageFactory_(factory),
+      requestTimeout_(requestTimeout)
+  {}
 
   void operator()(const RequestSlot& slot)
   {
     if(slot.isTimeout(requestTimeout_)) {
-      if(logger_->debug()) {
-        logger_->debug(MSG_DELETING_REQUEST_SLOT_TIMEOUT,
-                       util::itos(cuid_).c_str(),
-                       slot.getBlockIndex());
-        logger_->debug("index=%d, begin=%d", slot.getIndex(), slot.getBegin());
-      }
+      A2_LOG_DEBUG(fmt(MSG_DELETING_REQUEST_SLOT_TIMEOUT,
+                       cuid_,
+                       static_cast<unsigned long>(slot.getIndex()),
+                       slot.getBegin(),
+                       static_cast<unsigned long>(slot.getBlockIndex())));
       slot.getPiece()->cancelBlock(slot.getBlockIndex());
       peer_->snubbing(true);
     } else if(slot.getPiece()->hasBlock(slot.getBlockIndex())) {
-      if(logger_->debug()) {
-        logger_->debug(MSG_DELETING_REQUEST_SLOT_ACQUIRED,
-                       util::itos(cuid_).c_str(),
-                       slot.getBlockIndex());
-        logger_->debug("index=%d, begin=%d", slot.getIndex(), slot.getBegin());
-      }
+      A2_LOG_DEBUG(fmt(MSG_DELETING_REQUEST_SLOT_ACQUIRED,
+                       cuid_,
+                       static_cast<unsigned long>(slot.getIndex()),
+                       slot.getBegin(),
+                       static_cast<unsigned long>(slot.getBlockIndex())));
       messageDispatcher_->addMessageToQueue
         (messageFactory_->createCancelMessage(slot.getIndex(),
                                               slot.getBegin(),
@@ -308,7 +302,7 @@ public:
     }
   }
 };
-}
+} // namespace
 
 namespace {
 class FindStaleRequestSlot {
@@ -334,12 +328,12 @@ public:
     }
   }
 };
-}
+} // namespace
 
 void DefaultBtMessageDispatcher::checkRequestSlotAndDoNecessaryThing()
 {
   std::for_each(requestSlots_.begin(), requestSlots_.end(),
-                ProcessStaleRequestSlot(cuid,
+                ProcessStaleRequestSlot(cuid_,
                                         peer_,
                                         pieceStorage_,
                                         this,
@@ -372,7 +366,7 @@ public:
     }
   }
 };
-}
+} // namespace
 
 bool DefaultBtMessageDispatcher::isOutstandingRequest
 (size_t index, size_t blockIndex) {
@@ -407,7 +401,7 @@ void DefaultBtMessageDispatcher::removeOutstandingRequest
   std::deque<RequestSlot>::iterator i =
     std::lower_bound(requestSlots_.begin(), requestSlots_.end(), slot);
   if(i != requestSlots_.end() && (*i) == slot) {
-    AbortOutstandingRequest(slot.getPiece(), cuid)(*i);
+    AbortOutstandingRequest(slot.getPiece(), cuid_)(*i);
     requestSlots_.erase(i);
   }
 }
@@ -451,14 +445,12 @@ void DefaultBtMessageDispatcher::setPeerStorage
   peerStorage_ = peerStorage;
 }
 
-void DefaultBtMessageDispatcher::setBtMessageFactory
-(const WeakHandle<BtMessageFactory>& factory)
+void DefaultBtMessageDispatcher::setBtMessageFactory(BtMessageFactory* factory)
 {
   messageFactory_ = factory;
 }
 
-void DefaultBtMessageDispatcher::setRequestGroupMan
-(const WeakHandle<RequestGroupMan>& rgman)
+void DefaultBtMessageDispatcher::setRequestGroupMan(RequestGroupMan* rgman)
 {
   requestGroupMan_ = rgman;
 }
