@@ -32,14 +32,15 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_DEFAULT_BT_MESSAGE_DISPATCHER_H_
-#define _D_DEFAULT_BT_MESSAGE_DISPATCHER_H_
+#ifndef D_DEFAULT_BT_MESSAGE_DISPATCHER_H
+#define D_DEFAULT_BT_MESSAGE_DISPATCHER_H
 
 #include "BtMessageDispatcher.h"
 
 #include <deque>
 
 #include "a2time.h"
+#include "Command.h"
 
 namespace aria2 {
 
@@ -50,22 +51,20 @@ class BtMessage;
 class BtMessageFactory;
 class Peer;
 class Piece;
-class Logger;
 class RequestGroupMan;
 
 class DefaultBtMessageDispatcher : public BtMessageDispatcher {
 private:
-  int32_t cuid;
-  std::deque<SharedHandle<BtMessage> > messageQueue;
-  std::deque<RequestSlot> requestSlots;
-  SharedHandle<DownloadContext> _downloadContext;
-  SharedHandle<PeerStorage> _peerStorage;
-  SharedHandle<PieceStorage> _pieceStorage;
-  WeakHandle<BtMessageFactory> messageFactory;
-  SharedHandle<Peer> peer;
-  WeakHandle<RequestGroupMan> _requestGroupMan;
-  time_t requestTimeout;
-  Logger* logger;
+  cuid_t cuid_;
+  std::deque<SharedHandle<BtMessage> > messageQueue_;
+  std::deque<RequestSlot> requestSlots_;
+  SharedHandle<DownloadContext> downloadContext_;
+  SharedHandle<PeerStorage> peerStorage_;
+  SharedHandle<PieceStorage> pieceStorage_;
+  BtMessageFactory* messageFactory_;
+  SharedHandle<Peer> peer_;
+  RequestGroupMan* requestGroupMan_;
+  time_t requestTimeout_;
 public:
   DefaultBtMessageDispatcher();
 
@@ -93,12 +92,12 @@ public:
   virtual bool isSendingInProgress();
 
   virtual size_t countMessageInQueue() {
-    return messageQueue.size();
+    return messageQueue_.size();
   }
 
   virtual size_t countOutstandingRequest()
   {
-    return requestSlots.size();
+    return requestSlots_.size();
   }
   
   virtual bool isOutstandingRequest(size_t index, size_t blockIndex);
@@ -113,12 +112,12 @@ public:
 
   const std::deque<SharedHandle<BtMessage> >& getMessageQueue() const
   {
-    return messageQueue;
+    return messageQueue_;
   }
 
   const std::deque<RequestSlot>& getRequestSlots() const
   {
-    return requestSlots;
+    return requestSlots_;
   }
 
   void setPeer(const SharedHandle<Peer>& peer);
@@ -129,16 +128,18 @@ public:
 
   void setPeerStorage(const SharedHandle<PeerStorage>& peerStorage);
 
-  void setBtMessageFactory(const WeakHandle<BtMessageFactory>& factory);
+  void setBtMessageFactory(BtMessageFactory* factory);
 
-  void setRequestGroupMan(const WeakHandle<RequestGroupMan>& rgman);
+  void setRequestGroupMan(RequestGroupMan* rgman);
 
-  void setCuid(int32_t cuid) {
-    this->cuid = cuid;
+  void setCuid(cuid_t cuid)
+  {
+    cuid_ = cuid;
   }
 
-  void setRequestTimeout(time_t requestTimeout) {
-    this->requestTimeout = requestTimeout;
+  void setRequestTimeout(time_t requestTimeout)
+  {
+    requestTimeout_ = requestTimeout;
   }
 };
 
@@ -146,4 +147,4 @@ typedef SharedHandle<DefaultBtMessageDispatcher> DefaultBtMessageDispatcherHandl
 
 } // namespace aria2
 
-#endif // _D_DEFAULT_BT_MESSAGE_DISPATCHER_H_
+#endif // D_DEFAULT_BT_MESSAGE_DISPATCHER_H
