@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_FILE_ENTRY_H_
-#define _D_FILE_ENTRY_H_
+#ifndef D_FILE_ENTRY_H
+#define D_FILE_ENTRY_H
 
 #include "common.h"
 
@@ -46,15 +46,14 @@
 #include "File.h"
 #include "Request.h"
 #include "URIResult.h"
-#include "DownloadResultCode.h"
+#include "error_code.h"
 #include "A2STR.h"
+#include "TimerA2.h"
 
 namespace aria2 {
 
 class URISelector;
 
-class Logger;
-
 class FileEntry {
 private:
   std::string path_;
@@ -69,9 +68,10 @@ private:
   // URIResult is stored in the ascending order of the time when its result is
   // available.
   std::deque<URIResult> uriResults_;
-  bool singleHostMultiConnection_;
+  bool uniqueProtocol_;
+  size_t maxConnectionPerServer_;
   std::string originalName_;
-  Logger* logger_;
+  Timer lastFasterReplace_;
 
   void storePool(const SharedHandle<Request>& request);
 public:
@@ -90,7 +90,7 @@ public:
 
   const std::string& getPath() const { return path_; }
 
-  void setPath(const std::string& path) { path_ = path; }
+  void setPath(const std::string& path);
 
   uint64_t getLength() const { return length_; }
 
@@ -106,8 +106,6 @@ public:
 
   void setRequested(bool flag) { requested_ = flag; }
 
-  void setupDir();
-
   const std::deque<std::string>& getRemainingUris() const
   {
     return uris_;
@@ -144,15 +142,10 @@ public:
   // Inserts uris_ and spentUris_ into uris.
   void getUris(std::vector<std::string>& uris) const;
 
-  void setContentType(const std::string& contentType)
-  {
-    contentType_ = contentType;
-  }
+  void setContentType(const std::string& contentType);
 
   const std::string& getContentType() const { return contentType_; }
 
-  std::string selectUri(const SharedHandle<URISelector>& uriSelector);
-
   // If pooled Request object is available, one of them is removed
   // from the pool and returned.  If pool is empty, then select URI
   // using selectUri(selector) and construct Request object using it
@@ -160,9 +153,14 @@ public:
   // newly created Request. If Request object is retrieved from the
   // pool, referer is ignored.  If method is given, it is set to newly
   // created Request. If Request object is retrieved from the pool,
-  // method is ignored.
+  // method is ignored. If uriReuse is true and selector does not
+  // returns Request object either because uris_ is empty or all URI
+  // are not be usable because maxConnectionPerServer_ limit, then
+  // reuse used URIs and do selection again.
   SharedHandle<Request> getRequest
   (const SharedHandle<URISelector>& selector,
+   bool uriReuse,
+   const std::vector<std::pair<size_t, std::string> >& usedHosts,
    const std::string& referer = A2STR::NIL,
    const std::string& method = Request::METHOD_GET);
 
@@ -175,15 +173,9 @@ public:
 
   bool removeRequest(const SharedHandle<Request>& request);
 
-  size_t countInFlightRequest() const
-  {
-    return inFlightRequests_.size();
-  }
+  size_t countInFlightRequest() const;
 
-  size_t countPooledRequest() const
-  {
-    return requestPool_.size();
-  }
+  size_t countPooledRequest() const;
 
   const std::deque<SharedHandle<Request> >& getInFlightRequests() const
   {
@@ -201,7 +193,7 @@ public:
 
   void removeIdenticalURI(const std::string& uri);
 
-  void addURIResult(std::string uri, downloadresultcode::RESULT result);
+  void addURIResult(std::string uri, error_code::Value result);
 
   const std::deque<URIResult>& getURIResults() const
   {
@@ -211,30 +203,26 @@ public:
   // Extracts URIResult whose _result is r and stores them into res.
   // The extracted URIResults are removed from uriResults_.
   void extractURIResult
-  (std::deque<URIResult>& res, downloadresultcode::RESULT r);
+  (std::deque<URIResult>& res, error_code::Value r);
 
-  void disableSingleHostMultiConnection()
+  void setMaxConnectionPerServer(size_t n)
   {
-    singleHostMultiConnection_ = false;
+    maxConnectionPerServer_ = n;
   }
 
-  bool isSingleHostMultiConnectionEnabled() const
+  size_t getMaxConnectionPerServer() const
   {
-    return singleHostMultiConnection_;
+    return maxConnectionPerServer_;
   }
 
-  // Reuse URIs which have not emitted error so far.  Thie method
-  // tries to reuse at least num URIs.  If less than num URIs found to
-  // reuse, those URIs are used more than once so that num URIs total
-  // are available to reuse.
-  void reuseUri(size_t num);
+  // Reuse URIs which have not emitted error so far and whose host
+  // component is not included in ignore. The reusable URIs are
+  // appended to uris_ maxConnectionPerServer_ times.
+  void reuseUri(const std::vector<std::string>& ignore);
 
   void releaseRuntimeResource();
 
-  void setOriginalName(const std::string& originalName)
-  {
-    originalName_ = originalName;
-  }
+  void setOriginalName(const std::string& originalName);
 
   const std::string& getOriginalName() const
   {
@@ -243,9 +231,16 @@ public:
 
   bool removeUri(const std::string& uri);
 
-  bool emptyRequestUri() const
+  bool emptyRequestUri() const;
+
+  void setUniqueProtocol(bool f)
   {
-    return uris_.empty() && inFlightRequests_.empty() && requestPool_.empty();
+    uniqueProtocol_ = f;
+  }
+
+  bool isUniqueProtocol() const
+  {
+    return uniqueProtocol_;
   }
 };
 
@@ -300,7 +295,7 @@ void writeFilePath
 (InputIterator first, InputIterator last, std::ostream& o, bool memory)
 {
   SharedHandle<FileEntry> e = getFirstRequestedFileEntry(first, last);
-  if(e.isNull()) {
+  if(!e) {
     o << "n/a";
   } else {
     if(e->getPath().empty()) {
@@ -325,6 +320,6 @@ void writeFilePath
   }
 }
 
-}
+} // namespace aria2
 
-#endif // _D_FILE_ENTRY_H_
+#endif // D_FILE_ENTRY_H
