@@ -32,8 +32,8 @@
  * files in the program, then also delete it here.
  */
 /* copyright --> */
-#ifndef _D_GZIP_ENCODER_H_
-#define _D_GZIP_ENCODER_H_
+#ifndef D_GZIP_ENCODER_H
+#define D_GZIP_ENCODER_H
 #include "common.h"
 
 #include <string>
@@ -52,6 +52,8 @@ private:
   std::string internalBuf_;
 
   std::string encode(const unsigned char* in, size_t length, int flush);
+  // Not implemented
+  GZipEncoder& operator<<(char c);
 public:
   GZipEncoder();
 
@@ -68,6 +70,10 @@ public:
   // in this class.
   GZipEncoder& operator<<(const std::string& s);
 
+  // Feeds binary data in s with size length to deflater.  The
+  // deflated result is kept in this class.
+  GZipEncoder& write(const char* s, size_t length);
+
   // Feeds integer to deflator. Before passed to deflator, i is
   // converted to std::string using util::itos().  The deflated result
   // is kept in this class.
@@ -94,4 +100,4 @@ public:
 
 } // namespace aria2
 
-#endif // _D_GZIP_ENCODER_H_
+#endif // D_GZIP_ENCODER_H
