@@ -55,24 +55,31 @@
 #include "AuthConfigFactory.h"
 #include "AuthConfig.h"
 #include "a2functional.h"
+#include "fmt.h"
+#include "SocketRecvBuffer.h"
 
 namespace aria2 {
 
 HttpRequestEntry::HttpRequestEntry
-(const SharedHandle<HttpRequest>& httpRequest):
-  _httpRequest(httpRequest),
-  _proc(new HttpHeaderProcessor()) {}
+(const SharedHandle<HttpRequest>& httpRequest)
+  : httpRequest_(httpRequest),
+    proc_(new HttpHeaderProcessor())
+{}
 
 HttpRequestEntry::~HttpRequestEntry() {}
 
-HttpConnection::HttpConnection(int32_t cuid,
-                               const SocketHandle& socket,
-                               const Option* op):
-  cuid(cuid), socket(socket),
-  _socketBuffer(socket),
-  option(op), logger(LogFactory::getInstance())
+HttpConnection::HttpConnection
+(cuid_t cuid,
+ const SocketHandle& socket,
+ const SharedHandle<SocketRecvBuffer>& socketRecvBuffer)
+  : cuid_(cuid),
+    socket_(socket),
+    socketRecvBuffer_(socketRecvBuffer),
+    socketBuffer_(socket)
 {}
 
+HttpConnection::~HttpConnection() {}
+
 std::string HttpConnection::eraseConfidentialInfo(const std::string& request)
 {
   std::istringstream istr(request);
@@ -95,69 +102,71 @@ std::string HttpConnection::eraseConfidentialInfo(const std::string& request)
 void HttpConnection::sendRequest(const SharedHandle<HttpRequest>& httpRequest)
 {
   std::string request = httpRequest->createRequest();
-  logger->info(MSG_SENDING_REQUEST, cuid, eraseConfidentialInfo(request).c_str());
-  _socketBuffer.pushStr(request);
-  _socketBuffer.send();
+  A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                  cuid_,
+                  eraseConfidentialInfo(request).c_str()));
+  socketBuffer_.pushStr(request);
+  socketBuffer_.send();
   SharedHandle<HttpRequestEntry> entry(new HttpRequestEntry(httpRequest));
-  outstandingHttpRequests.push_back(entry);
+  outstandingHttpRequests_.push_back(entry);
 }
 
 void HttpConnection::sendProxyRequest
 (const SharedHandle<HttpRequest>& httpRequest)
 {
   std::string request = httpRequest->createProxyRequest();
-  logger->info(MSG_SENDING_REQUEST, cuid, eraseConfidentialInfo(request).c_str());
-  _socketBuffer.pushStr(request);
-  _socketBuffer.send();
+  A2_LOG_INFO(fmt(MSG_SENDING_REQUEST,
+                  cuid_,
+                  eraseConfidentialInfo(request).c_str()));
+  socketBuffer_.pushStr(request);
+  socketBuffer_.send();
   SharedHandle<HttpRequestEntry> entry(new HttpRequestEntry(httpRequest));
-  outstandingHttpRequests.push_back(entry);
+  outstandingHttpRequests_.push_back(entry);
 }
 
 SharedHandle<HttpResponse> HttpConnection::receiveResponse()
 {
-  if(outstandingHttpRequests.empty()) {
+  if(outstandingHttpRequests_.empty()) {
     throw DL_ABORT_EX(EX_NO_HTTP_REQUEST_ENTRY_FOUND);
   }
-  HttpRequestEntryHandle entry = outstandingHttpRequests.front();
+  HttpRequestEntryHandle entry = outstandingHttpRequests_.front();
   HttpHeaderProcessorHandle proc = entry->getHttpHeaderProcessor();
-
-  unsigned char buf[512];
-  size_t size = sizeof(buf);
-  socket->peekData(buf, size);
-  if(size == 0) {
-    if(socket->wantRead() || socket->wantWrite()) {
-      return SharedHandle<HttpResponse>();
-    } else {
-      throw DL_RETRY_EX(EX_INVALID_RESPONSE);
+  if(socketRecvBuffer_->bufferEmpty()) {
+    if(socketRecvBuffer_->recv() == 0 &&
+       !socket_->wantRead() && !socket_->wantWrite()) {
+      throw DL_RETRY_EX(EX_GOT_EOF);
     }
   }
-  proc->update(buf, size);
-  if(!proc->eoh()) {
-    socket->readData(buf, size);
-    return SharedHandle<HttpResponse>();
+  proc->update(socketRecvBuffer_->getBuffer(),
+               socketRecvBuffer_->getBufferLength());
+  SharedHandle<HttpResponse> httpResponse;
+  size_t shiftBufferLength;
+  if(proc->eoh()) {
+    SharedHandle<HttpHeader> httpHeader = proc->getHttpResponseHeader();
+    size_t putbackDataLength = proc->getPutBackDataLength();
+    A2_LOG_INFO(fmt(MSG_RECEIVE_RESPONSE,
+                    cuid_,
+                    proc->getHeaderString().c_str()));
+    assert(socketRecvBuffer_->getBufferLength() >= putbackDataLength);
+    shiftBufferLength = socketRecvBuffer_->getBufferLength()-putbackDataLength;
+    httpResponse.reset(new HttpResponse());
+    httpResponse->setCuid(cuid_);
+    httpResponse->setHttpHeader(httpHeader);
+    httpResponse->setHttpRequest(entry->getHttpRequest());
+    outstandingHttpRequests_.pop_front();
+  } else {
+    shiftBufferLength = socketRecvBuffer_->getBufferLength();
   }
-  size_t putbackDataLength = proc->getPutBackDataLength();
-  size -= putbackDataLength;
-  socket->readData(buf, size);
-
-  logger->info(MSG_RECEIVE_RESPONSE, cuid, proc->getHeaderString().c_str());
-  SharedHandle<HttpHeader> httpHeader = proc->getHttpResponseHeader();
-  SharedHandle<HttpResponse> httpResponse(new HttpResponse());
-  httpResponse->setCuid(cuid);
-  httpResponse->setHttpHeader(httpHeader);
-  httpResponse->setHttpRequest(entry->getHttpRequest());
-
-  outstandingHttpRequests.pop_front();
-
+  socketRecvBuffer_->shiftBuffer(shiftBufferLength);
   return httpResponse;
 }
 
 bool HttpConnection::isIssued(const SharedHandle<Segment>& segment) const
 {
-  for(HttpRequestEntries::const_iterator itr = outstandingHttpRequests.begin(),
-        eoi = outstandingHttpRequests.end(); itr != eoi; ++itr) {
+  for(HttpRequestEntries::const_iterator itr = outstandingHttpRequests_.begin(),
+        eoi = outstandingHttpRequests_.end(); itr != eoi; ++itr) {
     SharedHandle<HttpRequest> httpRequest = (*itr)->getHttpRequest();
-    if(httpRequest->getSegment() == segment) {
+    if(*httpRequest->getSegment() == *segment) {
       return true;
     }
   }
@@ -166,21 +175,21 @@ bool HttpConnection::isIssued(const SharedHandle<Segment>& segment) const
 
 SharedHandle<HttpRequest> HttpConnection::getFirstHttpRequest() const
 {
-  if(outstandingHttpRequests.empty()) {
+  if(outstandingHttpRequests_.empty()) {
     return SharedHandle<HttpRequest>();
   } else {
-    return outstandingHttpRequests.front()->getHttpRequest();
+    return outstandingHttpRequests_.front()->getHttpRequest();
   }
 }
 
 bool HttpConnection::sendBufferIsEmpty() const
 {
-  return _socketBuffer.sendBufferIsEmpty();
+  return socketBuffer_.sendBufferIsEmpty();
 }
 
 void HttpConnection::sendPendingData()
 {
-  _socketBuffer.send();
+  socketBuffer_.send();
 }
 
 } // namespace aria2
