@@ -176,11 +176,11 @@ std::string HttpRequest::createRequest()
   builtinHds.push_back(std::make_pair("Accept:", acceptTypes));
   if(contentEncodingEnabled_) {
     std::string acceptableEncodings;
-#ifdef HAVE_LIBZ
+#ifdef HAVE_ZLIB
     if(acceptGzip_) {
       acceptableEncodings += "deflate, gzip";
     }
-#endif // HAVE_LIBZ
+#endif // HAVE_ZLIB
     if(!acceptableEncodings.empty()) {
       builtinHds.push_back
         (std::make_pair("Accept-Encoding:", acceptableEncodings));
@@ -322,6 +322,11 @@ void HttpRequest::addHeader(const std::string& headersString)
   headers_.insert(headers_.end(), headers.begin(), headers.end());
 }
 
+void HttpRequest::clearHeader()
+{
+  headers_.clear();
+}
+
 void HttpRequest::addAcceptType(const std::string& type)
 {
   acceptTypes_.push_back(type);
@@ -431,4 +436,20 @@ void HttpRequest::setIfModifiedSinceHeader(const std::string& hd)
   ifModSinceHeader_ = hd;
 }
 
+bool HttpRequest::conditionalRequest() const
+{
+  if(!ifModSinceHeader_.empty()) {
+    return true;
+  }
+  for(std::vector<std::string>::const_iterator i = headers_.begin(),
+        eoi = headers_.end(); i != eoi; ++i) {
+    std::string hd = util::toLower(*i);
+    if(util::startsWith(hd, "if-modified-since") ||
+       util::startsWith(hd, "if-none-match")) {
+      return true;
+    }
+  }
+  return false;
+}
+
 } // namespace aria2
