@@ -46,45 +46,45 @@
 #include "Logger.h"
 #include "Base64.h"
 #include "a2functional.h"
+#include "fmt.h"
+#include "SocketRecvBuffer.h"
+#include "TimeA2.h"
 
 namespace aria2 {
 
-HttpServer::HttpServer(const SharedHandle<SocketCore>& socket,
-                       DownloadEngine* e):
-  socket_(socket),
-  socketBuffer_(socket),
-  e_(e),
-  headerProcessor_(new HttpHeaderProcessor()),
-  logger_(LogFactory::getInstance()),
-  keepAlive_(true),
-  gzip_(false),
-  acceptsPersistentConnection_(true),
-  acceptsGZip_(false)
+HttpServer::HttpServer
+(const SharedHandle<SocketCore>& socket,
+ DownloadEngine* e)
+ : socket_(socket),
+   socketRecvBuffer_(new SocketRecvBuffer(socket_)),
+   socketBuffer_(socket),
+   e_(e),
+   headerProcessor_(new HttpHeaderProcessor()),
+   keepAlive_(true),
+   gzip_(false),
+   acceptsPersistentConnection_(true),
+   acceptsGZip_(false)
 {}
 
 HttpServer::~HttpServer() {}
 
 SharedHandle<HttpHeader> HttpServer::receiveRequest()
 {
-  size_t size = 512;
-  unsigned char buf[size];
-  socket_->peekData(buf, size);
-  if(size == 0 && !(socket_->wantRead() || socket_->wantWrite())) {
-    throw DL_ABORT_EX(EX_EOF_FROM_PEER);
+  if(socketRecvBuffer_->bufferEmpty()) {
+    if(socketRecvBuffer_->recv() == 0 &&
+       !socket_->wantRead() && !socket_->wantWrite()) {
+      throw DL_ABORT_EX(EX_EOF_FROM_PEER);
+    }
   }
-  headerProcessor_->update(buf, size);
-  if(!headerProcessor_->eoh()) {
-    socket_->readData(buf, size);
-    return SharedHandle<HttpHeader>();
-  }
-  size_t putbackDataLength = headerProcessor_->getPutBackDataLength();
-  size -= putbackDataLength;
-  socket_->readData(buf, size);
-
-  SharedHandle<HttpHeader> header = headerProcessor_->getHttpRequestHeader();
-  if(!header.isNull()) {
-    logger_->info("HTTP Server received request\n%s",
-                  headerProcessor_->getHeaderString().c_str());
+  headerProcessor_->update(socketRecvBuffer_->getBuffer(),
+                           socketRecvBuffer_->getBufferLength());
+  if(headerProcessor_->eoh()) {
+    SharedHandle<HttpHeader> header = headerProcessor_->getHttpRequestHeader();
+    size_t putbackDataLength = headerProcessor_->getPutBackDataLength();
+    A2_LOG_INFO(fmt("HTTP Server received request\n%s",
+                    headerProcessor_->getHeaderString().c_str()));
+    socketRecvBuffer_->shiftBuffer
+      (socketRecvBuffer_->getBufferLength()-putbackDataLength);
     lastRequestHeader_ = header;
     lastBody_.clear();
     lastBody_.str("");
@@ -105,8 +105,11 @@ SharedHandle<HttpHeader> HttpServer::receiveRequest()
     acceptsGZip_ =
       std::find(acceptEncodings.begin(), acceptEncodings.end(), "gzip")
       != acceptEncodings.end();
+    return header;
+  } else {
+    socketRecvBuffer_->clearBuffer();
+    return SharedHandle<HttpHeader>();
   }
-  return header;
 }
 
 bool HttpServer::receiveBody()
@@ -114,16 +117,18 @@ bool HttpServer::receiveBody()
   if(lastContentLength_ == 0) {
     return true;
   }
-  const size_t BUFLEN = 4096;
-  char buf[BUFLEN];
-  size_t length = std::min(BUFLEN,
-                           static_cast<size_t>
-                           (lastContentLength_-lastBody_.tellg()));
-  socket_->readData(buf, length);
-  if(length == 0 && !(socket_->wantRead() || socket_->wantWrite())) {
-    throw DL_ABORT_EX(EX_EOF_FROM_PEER);
+  if(socketRecvBuffer_->bufferEmpty()) {
+    if(socketRecvBuffer_->recv() == 0 &&
+       !socket_->wantRead() && !socket_->wantWrite()) {
+      throw DL_ABORT_EX(EX_EOF_FROM_PEER);
+    }
   }
-  lastBody_.write(buf, length);
+  size_t length =
+    std::min(socketRecvBuffer_->getBufferLength(),
+             static_cast<size_t>(lastContentLength_-lastBody_.tellg()));
+  lastBody_.write(reinterpret_cast<const char*>(socketRecvBuffer_->getBuffer()),
+                  length);
+  socketRecvBuffer_->shiftBuffer(length);
   return lastContentLength_ == static_cast<uint64_t>(lastBody_.tellp());
 }
 
@@ -132,6 +137,11 @@ std::string HttpServer::getBody() const
   return lastBody_.str();
 }
 
+const std::string& HttpServer::getMethod() const
+{
+  return lastRequestHeader_->getMethod();
+}
+
 const std::string& HttpServer::getRequestPath() const
 {
   return lastRequestHeader_->getRequestPath();
@@ -147,10 +157,14 @@ void HttpServer::feedResponse(const std::string& status,
                               const std::string& text,
                               const std::string& contentType)
 {
+  std::string httpDate = Time().toHTTPDate();
   std::string header = "HTTP/1.1 ";
   strappend(header, status, "\r\n",
-            "Content-Type: ", contentType, "\r\n",
-            "Content-Length: ", util::uitos(text.size()), "\r\n");
+            "Date: ", httpDate, "\r\n",
+            "Content-Type: ", contentType, "\r\n");
+  strappend(header, "Content-Length: ", util::uitos(text.size()), "\r\n",
+            "Expires: ", httpDate, "\r\n",
+            "Cache-Control: no-cache\r\n");
   if(supportsGZip()) {
     header += "Content-Encoding: gzip\r\n";
   }
@@ -165,9 +179,7 @@ void HttpServer::feedResponse(const std::string& status,
   }
 
   header += "\r\n";
-  if(logger_->debug()) {
-    logger_->debug("HTTP Server sends response:\n%s", header.c_str());
-  }
+  A2_LOG_DEBUG(fmt("HTTP Server sends response:\n%s", header.c_str()));
   socketBuffer_.pushStr(header);
   socketBuffer_.pushStr(text);
 }
@@ -203,4 +215,11 @@ bool HttpServer::authenticate()
   return username_ == userpassPair.first && password_ == userpassPair.second;
 }
 
+void HttpServer::setUsernamePassword
+(const std::string& username, const std::string& password)
+{
+  username_ = username;
+  password_ = password;
+}
+
 } // namespace aria2
