@@ -49,11 +49,12 @@
 #include "message.h"
 #include "DlRetryEx.h"
 #include "DlAbortEx.h"
-#include "StringFormat.h"
+#include "fmt.h"
 #include "util.h"
 #include "TimeA2.h"
 #include "a2functional.h"
 #include "LogFactory.h"
+#include "A2STR.h"
 #ifdef ENABLE_SSL
 # include "TLSContext.h"
 #endif // ENABLE_SSL
@@ -91,32 +92,29 @@ namespace aria2 {
 # define CLOSE(X) while(close(X) == -1 && errno == EINTR)
 #endif // __MINGW32__
 
-static const char *errorMsg(const int err)
+namespace {
+std::string errorMsg(int errNum)
 {
 #ifndef __MINGW32__
-  return strerror(err);
+  return util::safeStrerror(errNum);
 #else
   static char buf[256];
   if (FormatMessage(
                     FORMAT_MESSAGE_FROM_SYSTEM |
                     FORMAT_MESSAGE_IGNORE_INSERTS,
                     NULL,
-                    err,
+                    errNum,
                     MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // Default language
                     (LPTSTR) &buf,
                     sizeof(buf),
                     NULL
                     ) == 0) {
-    snprintf(buf, sizeof(buf), EX_SOCKET_UNKNOWN_ERROR, err, err);
+    snprintf(buf, sizeof(buf), EX_SOCKET_UNKNOWN_ERROR, errNum, errNum);
   }
   return buf;
 #endif // __MINGW32__
 }
-
-static const char *errorMsg()
-{
-  return errorMsg(SOCKET_ERRNO);
-}
+} // namespace
 
 int SocketCore::protocolFamily_ = AF_UNSPEC;
 
@@ -132,11 +130,17 @@ void SocketCore::setTLSContext(const SharedHandle<TLSContext>& tlsContext)
 }
 #endif // ENABLE_SSL
 
-SocketCore::SocketCore(int sockType):sockType_(sockType), sockfd_(-1)  {
+SocketCore::SocketCore(int sockType)
+  : sockType_(sockType),
+    sockfd_(-1)
+{
   init();
 }
 
-SocketCore::SocketCore(sock_t sockfd, int sockType):sockType_(sockType), sockfd_(sockfd) {
+SocketCore::SocketCore(sock_t sockfd, int sockType)
+  : sockType_(sockType),
+    sockfd_(sockfd)
+{
   init();
 }
 
@@ -148,55 +152,38 @@ void SocketCore::init()
   wantRead_ = false;
   wantWrite_ = false;
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
   // for SSL
   ssl = NULL;
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
   sslSession_ = 0;
-  peekBufMax_ = 4096;
-  peekBuf_ = 0;
-  peekBufLength_ = 0;
 #endif //HAVE_LIBGNUTLS
 }
 
 SocketCore::~SocketCore() {
   closeConnection();
-#ifdef HAVE_LIBGNUTLS
-  delete [] peekBuf_;
-#endif // HAVE_LIBGNUTLS
-}
-
-template<typename T>
-std::string uitos(T value)
-{
-  std::string str;
-  if(value == 0) {
-    str = "0";
-    return str;
-  }
-  while(value) {
-    char digit = value%10+'0';
-    str.insert(str.begin(), digit);
-    value /= 10;
-  }
-  return str;
 }
 
 void SocketCore::create(int family, int protocol)
 {
+  int errNum;
   closeConnection();
   sock_t fd = socket(family, sockType_, protocol);
+  errNum = SOCKET_ERRNO;
   if(fd == (sock_t) -1) {
     throw DL_ABORT_EX
-      (StringFormat("Failed to create socket. Cause:%s", errorMsg()).str());
+      (fmt("Failed to create socket. Cause:%s",
+           errorMsg(errNum).c_str()));
   }
   int sockopt = 1;
   if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR,
                 (a2_sockopt_t) &sockopt, sizeof(sockopt)) < 0) {
+    errNum = SOCKET_ERRNO;
     CLOSE(fd);
     throw DL_ABORT_EX
-      (StringFormat("Failed to create socket. Cause:%s", errorMsg()).str());
+      (fmt("Failed to create socket. Cause:%s",
+           errorMsg(errNum).c_str()));
   }
   sockfd_ = fd;
 }
@@ -205,18 +192,36 @@ static sock_t bindInternal(int family, int socktype, int protocol,
                            const struct sockaddr* addr, socklen_t addrlen,
                            std::string& error)
 {
+  int errNum;
   sock_t fd = socket(family, socktype, protocol);
+  errNum = SOCKET_ERRNO;
   if(fd == (sock_t) -1) {
+    error = errorMsg(errNum);
     return -1;
   }
   int sockopt = 1;
   if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (a2_sockopt_t) &sockopt,
                 sizeof(sockopt)) < 0) {
+    errNum = SOCKET_ERRNO;
+    error = errorMsg(errNum);
     CLOSE(fd);
     return -1;
   }
+#ifdef IPV6_V6ONLY
+  if(family == AF_INET6) {
+    int sockopt = 1;
+    if(setsockopt(fd, IPPROTO_IPV6, IPV6_V6ONLY, (a2_sockopt_t) &sockopt,
+                  sizeof(sockopt)) < 0) {
+      errNum = SOCKET_ERRNO;
+      error = errorMsg(errNum);
+      CLOSE(fd);
+      return -1;
+    }
+  }
+#endif // IPV6_V6ONLY
   if(::bind(fd, addr, addrlen) == -1) {
-    error = errorMsg();
+    errNum = SOCKET_ERRNO;
+    error = errorMsg(errNum);
     CLOSE(fd);
     return -1;
   }
@@ -228,8 +233,8 @@ static sock_t bindTo
  int getaddrinfoFlags, std::string& error)
 {
   struct addrinfo* res;
-  int s = callGetaddrinfo(&res, host, uitos(port).c_str(), family, sockType,
-                          getaddrinfoFlags, 0);  
+  int s = callGetaddrinfo(&res, host, util::uitos(port).c_str(),
+                          family, sockType, getaddrinfoFlags, 0);  
   if(s) {
     error = gai_strerror(s);
     return -1;
@@ -252,31 +257,25 @@ void SocketCore::bindWithFamily(uint16_t port, int family, int flags)
   std::string error;
   sock_t fd = bindTo(0, port, family, sockType_, flags, error);
   if(fd == (sock_t) -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_BIND, error.c_str()));
   } else {
     sockfd_ = fd;
   }
 }
 
-void SocketCore::bind(const std::string& addr, uint16_t port, int flags)
+void SocketCore::bind
+(const std::string& addr, uint16_t port, int family, int flags)
 {
   closeConnection();
   std::string error;
-  sock_t fd =
-    bindTo(addr.c_str(), port, protocolFamily_, sockType_, flags, error);
-  if(fd == (sock_t)-1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
+  const char* addrp;
+  if(addr.empty()) {
+    addrp = 0;
   } else {
-    sockfd_ = fd;
+    addrp = addr.c_str();
   }
-}
-
-void SocketCore::bind(uint16_t port, int flags)
-{
-  closeConnection();
-  std::string error;
   if(!(flags&AI_PASSIVE) || bindAddrs_.empty()) {
-    sock_t fd = bindTo(0, port, protocolFamily_, sockType_, flags, error);
+    sock_t fd = bindTo(addrp, port, family, sockType_, flags, error);
     if(fd != (sock_t) -1) {
       sockfd_ = fd;
     }
@@ -294,7 +293,11 @@ void SocketCore::bind(uint16_t port, int flags)
         error = gai_strerror(s);
         continue;
       }
-      sock_t fd = bindTo(host, port, protocolFamily_, sockType_, flags, error);
+      if(addrp && strcmp(host, addrp) != 0) {
+        error = "Given address and resolved address do not match.";
+        continue;
+      }
+      sock_t fd = bindTo(addrp, port, family, sockType_, flags, error);
       if(fd != (sock_t)-1) {
         sockfd_ = fd;
         break;
@@ -302,10 +305,15 @@ void SocketCore::bind(uint16_t port, int flags)
     }
   }
   if(sockfd_ == (sock_t) -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_BIND, error.c_str()));
   }
 }
 
+void SocketCore::bind(uint16_t port, int flags)
+{
+  bind(A2STR::NIL, port, protocolFamily_, flags);
+}
+
 void SocketCore::bind(const struct sockaddr* addr, socklen_t addrlen)
 {
   closeConnection();
@@ -314,14 +322,15 @@ void SocketCore::bind(const struct sockaddr* addr, socklen_t addrlen)
   if(fd != (sock_t)-1) {
     sockfd_ = fd;
   } else {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BIND, error.c_str()).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_BIND, error.c_str()));
   }
 }
 
 void SocketCore::beginListen()
 {
   if(listen(sockfd_, 1) == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_LISTEN, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_LISTEN, errorMsg(errNum).c_str()));
   }
 }
 
@@ -330,9 +339,12 @@ SocketCore* SocketCore::acceptConnection() const
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
   sock_t fd;
-  while((fd = accept(sockfd_, reinterpret_cast<struct sockaddr*>(&sockaddr), &len)) == (sock_t) -1 && SOCKET_ERRNO == A2_EINTR);
+  while((fd = accept(sockfd_, reinterpret_cast<struct sockaddr*>(&sockaddr),
+                     &len)) == (sock_t) -1 &&
+        SOCKET_ERRNO == A2_EINTR);
+  int errNum = SOCKET_ERRNO;
   if(fd == (sock_t) -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_ACCEPT, errorMsg()).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_ACCEPT, errorMsg(errNum).c_str()));
   }
   return new SocketCore(fd, sockType_);
 }
@@ -341,11 +353,27 @@ void SocketCore::getAddrInfo(std::pair<std::string, uint16_t>& addrinfo) const
 {
   struct sockaddr_storage sockaddr;
   socklen_t len = sizeof(sockaddr);
+  getAddrInfo(sockaddr, len);
+  addrinfo = util::getNumericNameInfo
+    (reinterpret_cast<const struct sockaddr*>(&sockaddr), len);
+}
+
+void SocketCore::getAddrInfo
+(struct sockaddr_storage& sockaddr, socklen_t& len) const
+{
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
   if(getsockname(sockfd_, addrp, &len) == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_GET_NAME, errorMsg(errNum).c_str()));
   }
-  addrinfo = util::getNumericNameInfo(addrp, len);
+}
+
+int SocketCore::getAddressFamily() const
+{
+  struct sockaddr_storage sockaddr;
+  socklen_t len = sizeof(sockaddr);
+  getAddrInfo(sockaddr, len);
+  return sockaddr.ss_family;
 }
 
 void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
@@ -354,7 +382,8 @@ void SocketCore::getPeerInfo(std::pair<std::string, uint16_t>& peerinfo) const
   socklen_t len = sizeof(sockaddr);
   struct sockaddr* addrp = reinterpret_cast<struct sockaddr*>(&sockaddr);
   if(getpeername(sockfd_, addrp, &len) == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_GET_NAME, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_GET_NAME, errorMsg(errNum).c_str()));
   }
   peerinfo = util::getNumericNameInfo(addrp, len);
 }
@@ -365,23 +394,28 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
   std::string error;
   struct addrinfo* res;
   int s;
-  s = callGetaddrinfo(&res, host.c_str(), uitos(port).c_str(), protocolFamily_,
-                      sockType_, 0, 0);
+  s = callGetaddrinfo(&res, host.c_str(), util::uitos(port).c_str(),
+                      protocolFamily_, sockType_, 0, 0);
   if(s) {
-    throw DL_ABORT_EX(StringFormat(EX_RESOLVE_HOSTNAME,
-                                   host.c_str(), gai_strerror(s)).str());
+    throw DL_ABORT_EX(fmt(EX_RESOLVE_HOSTNAME,
+                          host.c_str(),
+                          gai_strerror(s)));
   }
   WSAAPI_AUTO_DELETE<struct addrinfo*> resDeleter(res, freeaddrinfo);
   struct addrinfo* rp;
+  int errNum;
   for(rp = res; rp; rp = rp->ai_next) {
     sock_t fd = socket(rp->ai_family, rp->ai_socktype, rp->ai_protocol);
+    errNum = SOCKET_ERRNO;
     if(fd == (sock_t) -1) {
-      error = errorMsg();
+      error = errorMsg(errNum);
       continue;
     }
     int sockopt = 1;
-    if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (a2_sockopt_t) &sockopt, sizeof(sockopt)) < 0) {
-      error = errorMsg();
+    if(setsockopt(fd, SOL_SOCKET, SO_REUSEADDR, (a2_sockopt_t) &sockopt,
+                  sizeof(sockopt)) < 0) {
+      errNum = SOCKET_ERRNO;
+      error = errorMsg(errNum);
       CLOSE(fd);
       continue;
     }
@@ -392,10 +426,9 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
           i != eoi; ++i) {
         if(::bind(fd,reinterpret_cast<const struct sockaddr*>(&(*i).first),
                   (*i).second) == -1) {
-          error = errorMsg();
-          if(LogFactory::getInstance()->debug()) {
-            LogFactory::getInstance()->debug(EX_SOCKET_BIND, error.c_str());
-          }
+          errNum = SOCKET_ERRNO;
+          error = errorMsg(errNum);
+          A2_LOG_DEBUG(fmt(EX_SOCKET_BIND, error.c_str()));
         } else {
           bindSuccess = true;
           break;
@@ -412,7 +445,8 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
     setNonBlockingMode();
     if(connect(fd, rp->ai_addr, rp->ai_addrlen) == -1 &&
        SOCKET_ERRNO != A2_EINPROGRESS) {
-      error = errorMsg();
+      errNum = SOCKET_ERRNO;
+      error = errorMsg(errNum);
       CLOSE(sockfd_);
       sockfd_ = (sock_t) -1;
       continue;
@@ -422,8 +456,10 @@ void SocketCore::establishConnection(const std::string& host, uint16_t port)
     break;
   }
   if(sockfd_ == (sock_t) -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_CONNECT, host.c_str(),
-                                   error.c_str()).str());
+    throw DL_ABORT_EX
+      (fmt(EX_SOCKET_CONNECT,
+           host.c_str(),
+           error.c_str()));
   }
 }
 
@@ -431,7 +467,8 @@ void SocketCore::setSockOpt
 (int level, int optname, void* optval, socklen_t optlen)
 {
   if(setsockopt(sockfd_, level, optname, (a2_sockopt_t)optval, optlen) < 0) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_SET_OPT, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_SET_OPT, errorMsg(errNum).c_str()));
   }
 }   
 
@@ -442,8 +479,7 @@ void SocketCore::setMulticastInterface(const std::string& localAddr)
     addr.s_addr = htonl(INADDR_ANY);
   } else {
     if(inet_aton(localAddr.c_str(), &addr) == 0) {
-      throw DL_ABORT_EX
-        (StringFormat("inet_aton failed for %s", localAddr.c_str()).str());
+      throw DL_ABORT_EX(fmt("inet_aton failed for %s", localAddr.c_str()));
     }
   }
   setSockOpt(IPPROTO_IP, IP_MULTICAST_IF, &addr, sizeof(addr));
@@ -465,16 +501,14 @@ void SocketCore::joinMulticastGroup
 {
   in_addr multiAddr;
   if(inet_aton(multicastAddr.c_str(), &multiAddr) == 0) {
-    throw DL_ABORT_EX
-      (StringFormat("inet_aton failed for %s", multicastAddr.c_str()).str());
+    throw DL_ABORT_EX(fmt("inet_aton failed for %s", multicastAddr.c_str()));
   }
   in_addr ifAddr;
   if(localAddr.empty()) {
     ifAddr.s_addr = htonl(INADDR_ANY);
   } else {
     if(inet_aton(localAddr.c_str(), &ifAddr) == 0) {
-      throw DL_ABORT_EX
-        (StringFormat("inet_aton failed for %s", localAddr.c_str()).str());
+      throw DL_ABORT_EX(fmt("inet_aton failed for %s", localAddr.c_str()));
     }
   }
   struct ip_mreq mreq;
@@ -489,7 +523,8 @@ void SocketCore::setNonBlockingMode()
 #ifdef __MINGW32__
   static u_long flag = 1;
   if (::ioctlsocket(sockfd_, FIONBIO, &flag) == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_NONBLOCKING, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_NONBLOCKING, errorMsg(errNum).c_str()));
   }
 #else
   int flags;
@@ -505,7 +540,8 @@ void SocketCore::setBlockingMode()
 #ifdef __MINGW32__
   static u_long flag = 0;
   if (::ioctlsocket(sockfd_, FIONBIO, &flag) == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_BLOCKING, errorMsg()).str());
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX(fmt(EX_SOCKET_BLOCKING, errorMsg(errNum).c_str()));
   }
 #else
   int flags;
@@ -518,12 +554,12 @@ void SocketCore::setBlockingMode()
 
 void SocketCore::closeConnection()
 {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
   // for SSL
   if(secure_) {
     SSL_shutdown(ssl);
   }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
   if(secure_) {
     gnutls_bye(sslSession_, GNUTLS_SHUT_RDWR);
@@ -533,12 +569,12 @@ void SocketCore::closeConnection()
     CLOSE(sockfd_);
     sockfd_ = -1;
   }
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
   // for SSL
   if(secure_) {
     SSL_free(ssl);
   }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
   if(secure_) {
     gnutls_deinit(sslSession_);
@@ -563,13 +599,13 @@ bool SocketCore::isWritable(time_t timeout)
   p.events = POLLOUT;
   int r;
   while((r = poll(&p, 1, timeout*1000)) == -1 && errno == EINTR);
+  int errNum = SOCKET_ERRNO;
   if(r > 0) {
     return p.revents&(POLLOUT|POLLHUP|POLLERR);
   } else if(r == 0) {
     return false;
   } else {
-    throw DL_RETRY_EX
-      (StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
+    throw DL_RETRY_EX(fmt(EX_SOCKET_CHECK_WRITABLE, errorMsg(errNum).c_str()));
   }
 #else // !HAVE_POLL
 # ifndef __MINGW32__
@@ -584,17 +620,19 @@ bool SocketCore::isWritable(time_t timeout)
   tv.tv_usec = 0;
 
   int r = select(sockfd_+1, NULL, &fds, NULL, &tv);
+  int errNum = SOCKET_ERRNO;
   if(r == 1) {
     return true;
   } else if(r == 0) {
     // time out
     return false;
   } else {
-    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
+    if(errNum == A2_EINPROGRESS || errNum == A2_EINTR) {
       return false;
     } else {
       throw DL_RETRY_EX
-        (StringFormat(EX_SOCKET_CHECK_WRITABLE, errorMsg()).str());
+        (fmt(EX_SOCKET_CHECK_WRITABLE,
+             errorMsg(errNum).c_str()));
     }
   }
 #endif // !HAVE_POLL
@@ -602,24 +640,19 @@ bool SocketCore::isWritable(time_t timeout)
 
 bool SocketCore::isReadable(time_t timeout)
 {
-#ifdef HAVE_LIBGNUTLS
-  if(secure_ && peekBufLength_ > 0) {
-    return true;
-  }
-#endif // HAVE_LIBGNUTLS
 #ifdef HAVE_POLL
   struct pollfd p;
   p.fd = sockfd_;
   p.events = POLLIN;
   int r;
   while((r = poll(&p, 1, timeout*1000)) == -1 && errno == EINTR);
+  int errNum = SOCKET_ERRNO;
   if(r > 0) {
     return p.revents&(POLLIN|POLLHUP|POLLERR);
   } else if(r == 0) {
     return false;
   } else {
-    throw DL_RETRY_EX
-      (StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
+    throw DL_RETRY_EX(fmt(EX_SOCKET_CHECK_READABLE, errorMsg(errNum).c_str()));
   }
 #else // !HAVE_POLL
 # ifndef __MINGW32__
@@ -634,23 +667,25 @@ bool SocketCore::isReadable(time_t timeout)
   tv.tv_usec = 0;
 
   int r = select(sockfd_+1, &fds, NULL, NULL, &tv);
+  int errNum = SOCKET_ERRNO;
   if(r == 1) {
     return true;
   } else if(r == 0) {
     // time out
     return false;
   } else {
-    if(SOCKET_ERRNO == A2_EINPROGRESS || SOCKET_ERRNO == A2_EINTR) {
+    if(errNum == A2_EINPROGRESS || errNum == A2_EINTR) {
       return false;
     } else {
       throw DL_RETRY_EX
-        (StringFormat(EX_SOCKET_CHECK_READABLE, errorMsg()).str());
+        (fmt(EX_SOCKET_CHECK_READABLE,
+             errorMsg(errNum).c_str()));
     }
   }
 #endif // !HAVE_POLL
 }
 
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
 int SocketCore::sslHandleEAGAIN(int ret)
 {
   int error = SSL_get_error(ssl, ret);
@@ -664,7 +699,7 @@ int SocketCore::sslHandleEAGAIN(int ret)
   }
   return ret;
 }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 
 #ifdef HAVE_LIBGNUTLS
 void SocketCore::gnutlsRecordCheckDirection()
@@ -686,26 +721,27 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
 
   if(!secure_) {
     while((ret = send(sockfd_, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
+    int errNum = SOCKET_ERRNO;
     if(ret == -1) {
-      if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
+      if(A2_WOULDBLOCK(errNum)) {
         wantWrite_ = true;
         ret = 0;
       } else {
-        throw DL_RETRY_EX(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
+        throw DL_RETRY_EX(fmt(EX_SOCKET_SEND, errorMsg(errNum).c_str()));
       }
     }
   } else {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
     ret = SSL_write(ssl, data, len);
     if(ret < 0) {
       ret = sslHandleEAGAIN(ret);
     }
     if(ret < 0) {
       throw DL_RETRY_EX
-        (StringFormat
-         (EX_SOCKET_SEND, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
+        (fmt(EX_SOCKET_SEND,
+             ERR_error_string(SSL_get_error(ssl, ret), 0)));
     }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
     while((ret = gnutls_record_send(sslSession_, data, len)) ==
           GNUTLS_E_INTERRUPTED);
@@ -713,7 +749,7 @@ ssize_t SocketCore::writeData(const char* data, size_t len)
       gnutlsRecordCheckDirection();
       ret = 0;
     } else if(ret < 0) {
-      throw DL_RETRY_EX(StringFormat(EX_SOCKET_SEND, gnutls_strerror(ret)).str());
+      throw DL_RETRY_EX(fmt(EX_SOCKET_SEND, gnutls_strerror(ret)));
     }
 #endif // HAVE_LIBGNUTLS
   }
@@ -730,16 +766,17 @@ void SocketCore::readData(char* data, size_t& len)
   if(!secure_) {    
     while((ret = recv(sockfd_, data, len, 0)) == -1 && SOCKET_ERRNO == A2_EINTR);
     
+    int errNum = SOCKET_ERRNO;
     if(ret == -1) {
-      if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
+      if(A2_WOULDBLOCK(errNum)) {
         wantRead_ = true;
         ret = 0;
       } else {
-        throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
+        throw DL_RETRY_EX(fmt(EX_SOCKET_RECV, errorMsg(errNum).c_str()));
       }
     }
   } else {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
     // for SSL
     // TODO handling len == 0 case required
     ret = SSL_read(ssl, data, len);
@@ -748,18 +785,18 @@ void SocketCore::readData(char* data, size_t& len)
     }
     if(ret < 0) {
       throw DL_RETRY_EX
-        (StringFormat
-         (EX_SOCKET_RECV, ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
+        (fmt(EX_SOCKET_RECV,
+             ERR_error_string(SSL_get_error(ssl, ret), 0)));
     }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
-    ret = gnutlsRecv(data, len);
+    while((ret = gnutls_record_recv(sslSession_, data, len)) ==
+          GNUTLS_E_INTERRUPTED);
     if(ret == GNUTLS_E_AGAIN) {
       gnutlsRecordCheckDirection();
       ret = 0;
     } else if(ret < 0) {
-      throw DL_RETRY_EX
-        (StringFormat(EX_SOCKET_RECV, gnutls_strerror(ret)).str());
+      throw DL_RETRY_EX(fmt(EX_SOCKET_RECV, gnutls_strerror(ret)));
     }
 #endif // HAVE_LIBGNUTLS
   }
@@ -767,145 +804,23 @@ void SocketCore::readData(char* data, size_t& len)
   len = ret;
 }
 
-void SocketCore::peekData(char* data, size_t& len)
-{
-  ssize_t ret = 0;
-  wantRead_ = false;
-  wantWrite_ = false;
-
-  if(!secure_) {
-    while((ret = recv(sockfd_, data, len, MSG_PEEK)) == -1 &&
-          SOCKET_ERRNO == A2_EINTR);
-    if(ret == -1) {
-      if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
-        wantRead_ = true;
-        ret = 0;
-      } else {
-        throw DL_RETRY_EX(StringFormat(EX_SOCKET_PEEK, errorMsg()).str());
-      }
-    }
-  } else {
-#ifdef HAVE_LIBSSL
-    // for SSL
-    // TODO handling len == 0 case required
-    ret = SSL_peek(ssl, data, len);
-    if(ret < 0) {
-      ret = sslHandleEAGAIN(ret);
-    }
-    if(ret < 0) {
-      throw DL_RETRY_EX
-        (StringFormat(EX_SOCKET_PEEK,
-                      ERR_error_string(SSL_get_error(ssl, ret), 0)).str());
-    }
-#endif // HAVE_LIBSSL
-#ifdef HAVE_LIBGNUTLS
-    ret = gnutlsPeek(data, len);
-    if(ret == GNUTLS_E_AGAIN) {
-      gnutlsRecordCheckDirection();
-      ret = 0;
-    } else if(ret < 0) {
-      throw DL_RETRY_EX(StringFormat(EX_SOCKET_PEEK,
-                                     gnutls_strerror(ret)).str());
-    }
-#endif // HAVE_LIBGNUTLS
-  }
-
-  len = ret;
-}
-
-#ifdef HAVE_LIBGNUTLS
-size_t SocketCore::shiftPeekData(char* data, size_t len)
-{
-  if(peekBufLength_ <= len) {
-    memcpy(data, peekBuf_, peekBufLength_);
-    size_t ret = peekBufLength_;
-    peekBufLength_ = 0;
-    return ret;
-  } else {
-    memcpy(data, peekBuf_, len);
-    peekBufLength_ -= len;    
-    memmove(peekBuf_, peekBuf_+len, peekBufLength_);
-    return len;
-  }
-
-}
-
-void SocketCore::addPeekData(char* data, size_t len)
-{
-  if(peekBufLength_+len > peekBufMax_) {
-    char* temp = new char[peekBufMax_+len];
-    memcpy(temp, peekBuf_, peekBufLength_);
-    delete [] peekBuf_;
-    peekBuf_ = temp;
-    peekBufMax_ = peekBufLength_+len;
-  }
-  memcpy(peekBuf_+peekBufLength_, data, len);
-  peekBufLength_ += len;
-}
-
-static ssize_t GNUTLS_RECORD_RECV_NO_INTERRUPT
-(gnutls_session_t sslSession, char* data, size_t len)
-{
-  int ret;
-  while((ret = gnutls_record_recv(sslSession, data, len)) ==
-        GNUTLS_E_INTERRUPTED);
-  if(ret < 0 && ret != GNUTLS_E_AGAIN) {
-    throw DL_RETRY_EX
-      (StringFormat(EX_SOCKET_RECV, gnutls_strerror(ret)).str());
-  }
-  return ret;
-}
-
-ssize_t SocketCore::gnutlsRecv(char* data, size_t len)
-{
-  size_t plen = shiftPeekData(data, len);
-  if(plen < len) {
-    ssize_t ret = GNUTLS_RECORD_RECV_NO_INTERRUPT
-      (sslSession_, data+plen, len-plen);
-    if(ret == GNUTLS_E_AGAIN) {
-      return GNUTLS_E_AGAIN;
-    }
-    return plen+ret;
-  } else {
-    return plen;
-  }
-}
-
-ssize_t SocketCore::gnutlsPeek(char* data, size_t len)
-{
-  if(peekBufLength_ >= len) {
-    memcpy(data, peekBuf_, len);
-    return len;
-  } else {
-    memcpy(data, peekBuf_, peekBufLength_);
-    ssize_t ret = GNUTLS_RECORD_RECV_NO_INTERRUPT
-      (sslSession_, data+peekBufLength_, len-peekBufLength_);
-    if(ret == GNUTLS_E_AGAIN) {
-      return GNUTLS_E_AGAIN;
-    }
-    addPeekData(data+peekBufLength_, ret);
-    return peekBufLength_;
-  }
-}
-#endif // HAVE_LIBGNUTLS
-
 void SocketCore::prepareSecureConnection()
 {
   if(!secure_) {
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
     // for SSL
     ssl = SSL_new(tlsContext_->getSSLCtx());
     if(!ssl) {
       throw DL_ABORT_EX
-        (StringFormat(EX_SSL_INIT_FAILURE,
-                      ERR_error_string(ERR_get_error(), 0)).str());
+        (fmt(EX_SSL_INIT_FAILURE,
+             ERR_error_string(ERR_get_error(), 0)));
     }
     if(SSL_set_fd(ssl, sockfd_) == 0) {
       throw DL_ABORT_EX
-        (StringFormat(EX_SSL_INIT_FAILURE,
-                      ERR_error_string(ERR_get_error(), 0)).str());
+        (fmt(EX_SSL_INIT_FAILURE,
+             ERR_error_string(ERR_get_error(), 0)));
     }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
     int r;
     gnutls_init(&sslSession_, GNUTLS_CLIENT);
@@ -916,8 +831,7 @@ void SocketCore::prepareSecureConnection()
     // understand TLS1.1.
     r = gnutls_priority_set_direct(sslSession_, "NORMAL:!VERS-TLS1.1", &err);
     if(r != GNUTLS_E_SUCCESS) {
-      throw DL_ABORT_EX
-        (StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(r)).str());
+      throw DL_ABORT_EX(fmt(EX_SSL_INIT_FAILURE, gnutls_strerror(r)));
     }
     // put the x509 credentials to the current session
     gnutls_credentials_set(sslSession_, GNUTLS_CRD_CERTIFICATE,
@@ -933,7 +847,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
   if(secure_ == 1) {
     wantRead_ = false;
     wantWrite_ = false;
-#ifdef HAVE_LIBSSL
+#ifdef HAVE_OPENSSL
     int e = SSL_connect(ssl);
 
     if (e <= 0) {
@@ -951,8 +865,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       case SSL_ERROR_WANT_X509_LOOKUP:
       case SSL_ERROR_ZERO_RETURN:
         if (blocking_) {
-          throw DL_ABORT_EX
-            (StringFormat(EX_SSL_CONNECT_ERROR, ssl_error).str());
+          throw DL_ABORT_EX(fmt(EX_SSL_CONNECT_ERROR, ssl_error));
         }
         break;
 
@@ -963,8 +876,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         throw DL_ABORT_EX(EX_SSL_PROTOCOL_ERROR);
 
       default:
-        throw DL_ABORT_EX
-          (StringFormat(EX_SSL_UNKNOWN_ERROR, ssl_error).str());
+        throw DL_ABORT_EX(fmt(EX_SSL_UNKNOWN_ERROR, ssl_error));
       }
     }
     if(tlsContext_->peerVerificationEnabled()) {
@@ -978,8 +890,8 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       long verifyResult = SSL_get_verify_result(ssl);
       if(verifyResult != X509_V_OK) {
         throw DL_ABORT_EX
-          (StringFormat(MSG_CERT_VERIFICATION_FAILED,
-                        X509_verify_cert_error_string(verifyResult)).str());
+          (fmt(MSG_CERT_VERIFICATION_FAILED,
+               X509_verify_cert_error_string(verifyResult)));
       }
       X509_NAME* name = X509_get_subject_name(peerCert);
       if(!name) {
@@ -1010,15 +922,14 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         throw DL_ABORT_EX(MSG_HOSTNAME_NOT_MATCH);
       }
     }
-#endif // HAVE_LIBSSL
+#endif // HAVE_OPENSSL
 #ifdef HAVE_LIBGNUTLS
     int ret = gnutls_handshake(sslSession_);
     if(ret == GNUTLS_E_AGAIN) {
       gnutlsRecordCheckDirection();
       return false;
     } else if(ret < 0) {
-      throw DL_ABORT_EX
-        (StringFormat(EX_SSL_INIT_FAILURE, gnutls_strerror(ret)).str());
+      throw DL_ABORT_EX(fmt(EX_SSL_INIT_FAILURE, gnutls_strerror(ret)));
     }
 
     if(tlsContext_->peerVerificationEnabled()) {
@@ -1027,8 +938,8 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
       ret = gnutls_certificate_verify_peers2(sslSession_, &status);
       if(ret < 0) {
         throw DL_ABORT_EX
-          (StringFormat("gnutls_certificate_verify_peer2() failed. Cause: %s",
-                        gnutls_strerror(ret)).str());
+          (fmt("gnutls_certificate_verify_peer2() failed. Cause: %s",
+               gnutls_strerror(ret)));
       }
       if(status) {
         std::string errors;
@@ -1042,8 +953,7 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
           errors += " `issuer is not known'";
         }
         if(!errors.empty()) {
-          throw DL_ABORT_EX
-            (StringFormat(MSG_CERT_VERIFICATION_FAILED, errors.c_str()).str());
+          throw DL_ABORT_EX(fmt(MSG_CERT_VERIFICATION_FAILED, errors.c_str()));
         }
       }
       // certificate type: only X509 is allowed.
@@ -1063,16 +973,16 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         ret = gnutls_x509_crt_init(&cert);
         if(ret < 0) {
           throw DL_ABORT_EX
-            (StringFormat("gnutls_x509_crt_init() failed. Cause: %s",
-                          gnutls_strerror(ret)).str());
+            (fmt("gnutls_x509_crt_init() failed. Cause: %s",
+                 gnutls_strerror(ret)));
         }
         auto_delete<gnutls_x509_crt_t> certDeleter
           (cert, gnutls_x509_crt_deinit);
         ret = gnutls_x509_crt_import(cert, &peerCerts[i], GNUTLS_X509_FMT_DER);
         if(ret < 0) {
           throw DL_ABORT_EX
-            (StringFormat("gnutls_x509_crt_import() failed. Cause: %s",
-                          gnutls_strerror(ret)).str());
+            (fmt("gnutls_x509_crt_import() failed. Cause: %s",
+                 gnutls_strerror(ret)));
         }
         if(i == 0) {
           if(!gnutls_x509_crt_check_hostname(cert, hostname.c_str())) {
@@ -1095,7 +1005,6 @@ bool SocketCore::initiateSecureConnection(const std::string& hostname)
         }
       }
     }
-    peekBuf_ = new char[peekBufMax_];
 #endif // HAVE_LIBGNUTLS
     secure_ = 2;
     return true;
@@ -1112,28 +1021,30 @@ ssize_t SocketCore::writeData(const char* data, size_t len,
 
   struct addrinfo* res;
   int s;
-  s = callGetaddrinfo(&res, host.c_str(), uitos(port).c_str(), protocolFamily_,
-                      sockType_, 0, 0);
+  s = callGetaddrinfo(&res, host.c_str(), util::uitos(port).c_str(),
+                      protocolFamily_, sockType_, 0, 0);
   if(s) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_SEND, gai_strerror(s)).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_SEND, gai_strerror(s)));
   }
   WSAAPI_AUTO_DELETE<struct addrinfo*> resDeleter(res, freeaddrinfo);
   struct addrinfo* rp;
   ssize_t r = -1;
+  int errNum = 0;
   for(rp = res; rp; rp = rp->ai_next) {
     while((r = sendto(sockfd_, data, len, 0, rp->ai_addr, rp->ai_addrlen)) == -1
           && A2_EINTR == SOCKET_ERRNO);
+    errNum = SOCKET_ERRNO;
     if(r == static_cast<ssize_t>(len)) {
       break;
     }
-    if(r == -1 && A2_WOULDBLOCK(SOCKET_ERRNO)) {
+    if(r == -1 && A2_WOULDBLOCK(errNum)) {
       wantWrite_ = true;
       r = 0;
       break;
     }
   }
   if(r == -1) {
-    throw DL_ABORT_EX(StringFormat(EX_SOCKET_SEND, errorMsg()).str());
+    throw DL_ABORT_EX(fmt(EX_SOCKET_SEND, errorMsg(errNum).c_str()));
   }
   return r;
 }
@@ -1150,12 +1061,13 @@ ssize_t SocketCore::readDataFrom(char* data, size_t len,
   ssize_t r;
   while((r = recvfrom(sockfd_, data, len, 0, addrp, &sockaddrlen)) == -1 &&
         A2_EINTR == SOCKET_ERRNO);
+  int errNum = SOCKET_ERRNO;
   if(r == -1) {
-    if(A2_WOULDBLOCK(SOCKET_ERRNO)) {
+    if(A2_WOULDBLOCK(errNum)) {
       wantRead_ = true;
       r = 0;
     } else {
-      throw DL_RETRY_EX(StringFormat(EX_SOCKET_RECV, errorMsg()).str());
+      throw DL_RETRY_EX(fmt(EX_SOCKET_RECV, errorMsg(errNum).c_str()));
     }
   } else {
     sender = util::getNumericNameInfo(addrp, sockaddrlen);
@@ -1169,9 +1081,12 @@ std::string SocketCore::getSocketError() const
   int error;
   socklen_t optlen = sizeof(error);
 
-  if(getsockopt(sockfd_, SOL_SOCKET, SO_ERROR, (a2_sockopt_t) &error, &optlen) == -1) {
-    throw DL_ABORT_EX(StringFormat("Failed to get socket error: %s",
-                                   errorMsg()).str());
+  if(getsockopt(sockfd_, SOL_SOCKET, SO_ERROR,
+                (a2_sockopt_t) &error, &optlen) == -1) {
+    int errNum = SOCKET_ERRNO;
+    throw DL_ABORT_EX
+      (fmt("Failed to get socket error: %s",
+           errorMsg(errNum).c_str()));
   }
   if(error != 0) {
     return errorMsg(error);
@@ -1196,8 +1111,9 @@ void SocketCore::bindAddress(const std::string& iface)
   getInterfaceAddress(bindAddrs, iface, protocolFamily_);
   if(bindAddrs.empty()) {
     throw DL_ABORT_EX
-      (StringFormat(MSG_INTERFACE_NOT_FOUND,
-                    iface.c_str(), "not available").str());
+      (fmt(MSG_INTERFACE_NOT_FOUND,
+           iface.c_str(),
+           "not available"));
   } else {
     bindAddrs_ = bindAddrs;
     for(std::vector<std::pair<struct sockaddr_storage, socklen_t> >::
@@ -1210,9 +1126,7 @@ void SocketCore::bindAddress(const std::string& iface)
                       host, NI_MAXHOST, 0, 0,
                       NI_NUMERICHOST);
       if(s == 0) {
-        if(LogFactory::getInstance()->debug()) {
-          LogFactory::getInstance()->debug("Sockets will bind to %s", host);
-        }
+        A2_LOG_DEBUG(fmt("Sockets will bind to %s", host));
       }
     }
   }
@@ -1222,15 +1136,14 @@ void getInterfaceAddress
 (std::vector<std::pair<struct sockaddr_storage, socklen_t> >& ifAddrs,
  const std::string& iface, int family, int aiFlags)
 {
-  Logger* logger = LogFactory::getInstance();
-  if(logger->debug()) {
-    logger->debug("Finding interface %s", iface.c_str());
-  }
+  A2_LOG_DEBUG(fmt("Finding interface %s", iface.c_str()));
 #ifdef HAVE_GETIFADDRS
   // First find interface in interface addresses
   struct ifaddrs* ifaddr = 0;
   if(getifaddrs(&ifaddr) == -1) {
-    logger->info(MSG_INTERFACE_NOT_FOUND, iface.c_str(), errorMsg());
+    int errNum = SOCKET_ERRNO;
+    A2_LOG_INFO(fmt(MSG_INTERFACE_NOT_FOUND,
+                    iface.c_str(), errorMsg(errNum).c_str()));
   } else {
     auto_delete<struct ifaddrs*> ifaddrDeleter(ifaddr, freeifaddrs);
     for(struct ifaddrs* ifa = ifaddr; ifa; ifa = ifa->ifa_next) {
@@ -1269,7 +1182,7 @@ void getInterfaceAddress
     int s;
     s = callGetaddrinfo(&res, iface.c_str(), 0, family, SOCK_STREAM, aiFlags,0);
     if(s) {
-      logger->info(MSG_INTERFACE_NOT_FOUND, iface.c_str(), gai_strerror(s));
+      A2_LOG_INFO(fmt(MSG_INTERFACE_NOT_FOUND, iface.c_str(), gai_strerror(s)));
     } else {
       WSAAPI_AUTO_DELETE<struct addrinfo*> resDeleter(res, freeaddrinfo);
       struct addrinfo* rp;
@@ -1302,7 +1215,7 @@ int getDefaultAIFlags()
   return defaultAIFlags;
 }
 
-}
+} // namespace
 
 void setDefaultAIFlags(int flags)
 {
