@@ -39,8 +39,8 @@ namespace aria2 {
 /**
  * Constants
  */
-const std::string V_TRUE("true");
-const std::string V_FALSE("false");
+const std::string A2_V_TRUE("true");
+const std::string A2_V_FALSE("false");
 const std::string V_NONE("none");
 const std::string V_MEM("mem");
 const std::string V_ALL("all");
@@ -150,34 +150,76 @@ const std::string PREF_MAX_FILE_NOT_FOUND("max-file-not-found");
 // value: epoll | select
 const std::string PREF_EVENT_POLL("event-poll");
 const std::string V_EPOLL("epoll");
+const std::string V_KQUEUE("kqueue");
+const std::string V_PORT("port");
+const std::string V_POLL("poll");
 const std::string V_SELECT("select");
+// value: true | false
+const std::string PREF_ENABLE_RPC("enable-rpc");
 // value: 1*digit
-const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
+const std::string PREF_RPC_LISTEN_PORT("rpc-listen-port");
+// value: string
+const std::string PREF_RPC_USER("rpc-user");
+// value: string
+const std::string PREF_RPC_PASSWD("rpc-passwd");
+// value: 1*digit
+const std::string PREF_RPC_MAX_REQUEST_SIZE("rpc-max-request-size");
+// value: true | false
+const std::string PREF_RPC_LISTEN_ALL("rpc-listen-all");
 // value: true | false
 const std::string PREF_ENABLE_XML_RPC("enable-xml-rpc");
-// value: true | false
-const std::string PREF_DRY_RUN("dry-run");
-// value: true | false
-const std::string PREF_REUSE_URI("reuse-uri");
+// value: 1*digit
+const std::string PREF_XML_RPC_LISTEN_PORT("xml-rpc-listen-port");
 // value: string
 const std::string PREF_XML_RPC_USER("xml-rpc-user");
 // value: string
 const std::string PREF_XML_RPC_PASSWD("xml-rpc-passwd");
 // value: 1*digit
 const std::string PREF_XML_RPC_MAX_REQUEST_SIZE("xml-rpc-max-request-size");
+// value: true | false
+const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
+// value: true | false
+const std::string PREF_DRY_RUN("dry-run");
+// value: true | false
+const std::string PREF_REUSE_URI("reuse-uri");
 // value: string
 const std::string PREF_ON_DOWNLOAD_START("on-download-start");
+const std::string PREF_ON_DOWNLOAD_PAUSE("on-download-pause");
 const std::string PREF_ON_DOWNLOAD_STOP("on-download-stop");
 const std::string PREF_ON_DOWNLOAD_COMPLETE("on-download-complete");
 const std::string PREF_ON_DOWNLOAD_ERROR("on-download-error");
-// value: true | false
-const std::string PREF_XML_RPC_LISTEN_ALL("xml-rpc-listen-all");
 // value: string
 const std::string PREF_INTERFACE("interface");
 // value: true | false
 const std::string PREF_DISABLE_IPV6("disable-ipv6");
 // value: true | false
 const std::string PREF_HUMAN_READABLE("human-readable");
+// value: true | false
+const std::string PREF_REMOVE_CONTROL_FILE("remove-control-file");
+// value: true | false
+const std::string PREF_ALWAYS_RESUME("always-resume");
+// value: 1*digit
+const std::string PREF_MAX_RESUME_FAILURE_TRIES("max-resume-failure-tries");
+// value: string that your file system recognizes as a file name.
+const std::string PREF_SAVE_SESSION("save-session");
+// value: 1*digit
+const std::string PREF_MAX_CONNECTION_PER_SERVER("max-connection-per-server");
+// value: 1*digit
+const std::string PREF_MIN_SPLIT_SIZE("min-split-size");
+// value: true | false
+const std::string PREF_CONDITIONAL_GET("conditional-get");
+// value: true | false
+const std::string PREF_SELECT_LEAST_USED_HOST("select-least-used-host");
+// value: true | false
+const std::string PREF_ENABLE_ASYNC_DNS6("enable-async-dns6");
+// value: 1*digit
+const std::string PREF_MAX_DOWNLOAD_RESULT("max-download-result");
+// value: 1*digit
+const std::string PREF_RETRY_WAIT("retry-wait");
+// value: string
+const std::string PREF_ASYNC_DNS_SERVER("async-dns-server");
+// value: true | false
+const std::string PREF_SHOW_CONSOLE_READOUT("show-console-readout");
 
 /**
  * FTP related preferences
@@ -226,6 +268,8 @@ const std::string PREF_USE_HEAD("use-head");
 const std::string PREF_HTTP_AUTH_CHALLENGE("http-auth-challenge");
 // value: true | false
 const std::string PREF_HTTP_NO_CACHE("http-no-cache");
+// value: true | false
+const std::string PREF_HTTP_ACCEPT_GZIP("http-accept-gzip");
 
 /** 
  * Proxy related preferences
@@ -284,6 +328,8 @@ const std::string PREF_PEER_ID_PREFIX("peer-id-prefix");
 const std::string PREF_ENABLE_PEER_EXCHANGE("enable-peer-exchange");
 // values: true | false
 const std::string PREF_ENABLE_DHT("enable-dht");
+// values: a string
+const std::string PREF_DHT_LISTEN_ADDR("dht-listen-addr");
 // values: 1*digit
 const std::string PREF_DHT_LISTEN_PORT("dht-listen-port");
 // values: a string
@@ -294,6 +340,18 @@ const std::string PREF_DHT_ENTRY_POINT_PORT("dht-entry-point-port");
 const std::string PREF_DHT_ENTRY_POINT("dht-entry-point");
 // values: a string
 const std::string PREF_DHT_FILE_PATH("dht-file-path");
+// values: true | false
+const std::string PREF_ENABLE_DHT6("enable-dht6");
+// values: a string
+const std::string PREF_DHT_LISTEN_ADDR6("dht-listen-addr6");
+// values: a string
+const std::string PREF_DHT_ENTRY_POINT_HOST6("dht-entry-point-host6");
+// values: 1*digit
+const std::string PREF_DHT_ENTRY_POINT_PORT6("dht-entry-point-port6");
+// values: a string (hostname:port)
+const std::string PREF_DHT_ENTRY_POINT6("dht-entry-point6");
+// values: a string
+const std::string PREF_DHT_FILE_PATH6("dht-file-path6");
 // values: plain | arc4
 const std::string PREF_BT_MIN_CRYPTO_LEVEL("bt-min-crypto-level");
 const std::string V_PLAIN("plain");
@@ -328,6 +386,18 @@ const std::string PREF_BT_METADATA_ONLY("bt-metadata-only");
 const std::string PREF_BT_ENABLE_LPD("bt-enable-lpd");
 // values: string
 const std::string PREF_BT_LPD_INTERFACE("bt-lpd-interface");
+// values: 1*digit
+const std::string PREF_BT_TRACKER_TIMEOUT("bt-tracker-timeout");
+// values: 1*digit
+const std::string PREF_BT_TRACKER_CONNECT_TIMEOUT("bt-tracker-connect-timeout");
+// values: 1*digit
+const std::string PREF_DHT_MESSAGE_TIMEOUT("dht-message-timeout");
+// values: string
+const std::string PREF_ON_BT_DOWNLOAD_COMPLETE("on-bt-download-complete");
+// values: string
+const std::string PREF_BT_TRACKER("bt-tracker");
+// values: string
+const std::string PREF_BT_EXCLUDE_TRACKER("bt-exclude-tracker");
 
 /**
  * Metalink related preferences
