@@ -9,9 +9,7 @@
 #include "Option.h"
 #include "RequestGroup.h"
 #include "FileEntry.h"
-#ifdef ENABLE_MESSAGE_DIGEST
-# include "messageDigest.h"
-#endif // ENABLE_MESSAGE_DIGEST
+#include "Signature.h"
 
 namespace aria2 {
 
@@ -44,7 +42,7 @@ void Metalink2RequestGroupTest::testGenerate()
 {
   std::vector<SharedHandle<RequestGroup> > groups;
   option_->put(PREF_DIR, "/tmp");
-  Metalink2RequestGroup().generate(groups, "test.xml", option_);
+  Metalink2RequestGroup().generate(groups, A2_TEST_DIR"/test.xml", option_);
   // first file
   {
     SharedHandle<RequestGroup> rg = groups[0];
@@ -59,17 +57,15 @@ void Metalink2RequestGroupTest::testGenerate()
 
     const SharedHandle<DownloadContext>& dctx = rg->getDownloadContext();
 
-    CPPUNIT_ASSERT(!dctx.isNull());
+    CPPUNIT_ASSERT(dctx);
     CPPUNIT_ASSERT_EQUAL((uint64_t)0ULL, dctx->getTotalLength());
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), dctx->getDir());
 #ifdef ENABLE_MESSAGE_DIGEST
-    CPPUNIT_ASSERT_EQUAL(MessageDigestContext::SHA1,
-                         dctx->getChecksumHashAlgo());
+    CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), dctx->getChecksumHashAlgo());
     CPPUNIT_ASSERT_EQUAL
       (std::string("a96cf3f0266b91d87d5124cf94326422800b627d"),
        dctx->getChecksum());
 #endif // ENABLE_MESSAGE_DIGEST
-    CPPUNIT_ASSERT(!dctx->getSignature().isNull());
+    CPPUNIT_ASSERT(dctx->getSignature());
     CPPUNIT_ASSERT_EQUAL(std::string("pgp"), dctx->getSignature()->getType());
   }
   // second file
@@ -81,19 +77,17 @@ void Metalink2RequestGroupTest::testGenerate()
 
     const SharedHandle<DownloadContext>& dctx = rg->getDownloadContext();
 
-    CPPUNIT_ASSERT(!dctx.isNull());
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), dctx->getDir());
+    CPPUNIT_ASSERT(dctx);
 #ifdef ENABLE_MESSAGE_DIGEST
-    CPPUNIT_ASSERT_EQUAL(MessageDigestContext::SHA1, dctx->getPieceHashAlgo());
+    CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), dctx->getPieceHashAlgo());
     CPPUNIT_ASSERT_EQUAL((size_t)2, dctx->getPieceHashes().size());
     CPPUNIT_ASSERT_EQUAL((size_t)262144, dctx->getPieceLength());
-    CPPUNIT_ASSERT_EQUAL(MessageDigestContext::SHA1,
-                         dctx->getChecksumHashAlgo());
+    CPPUNIT_ASSERT_EQUAL(std::string("sha-1"), dctx->getChecksumHashAlgo());
     CPPUNIT_ASSERT_EQUAL
       (std::string("4c255b0ed130f5ea880f0aa061c3da0487e251cc"),
        dctx->getChecksum());
 #endif // ENABLE_MESSAGE_DIGEST
-    CPPUNIT_ASSERT(dctx->getSignature().isNull());
+    CPPUNIT_ASSERT(!dctx->getSignature());
   }
 
 #ifdef ENABLE_BITTORRENT
@@ -107,7 +101,7 @@ void Metalink2RequestGroupTest::testGenerate()
       (std::string("http://host/torrent-http.integrated.torrent"), uris[0]);
     const SharedHandle<DownloadContext>& dctx = rg->getDownloadContext();
 
-    CPPUNIT_ASSERT(!dctx.isNull());
+    CPPUNIT_ASSERT(dctx);
     CPPUNIT_ASSERT_EQUAL(groups[5]->getGID(), rg->belongsTo());
   }
 #endif // ENABLE_BITTORRENT
@@ -127,15 +121,15 @@ void Metalink2RequestGroupTest::testGenerate()
 
     const SharedHandle<DownloadContext>& dctx = rg->getDownloadContext();
 
-    CPPUNIT_ASSERT(!dctx.isNull());
-    CPPUNIT_ASSERT_EQUAL(std::string("/tmp"), dctx->getDir());
+    CPPUNIT_ASSERT(dctx);
   }
 }
 
 void Metalink2RequestGroupTest::testGenerate_groupByMetaurl()
 {
   std::vector<SharedHandle<RequestGroup> > groups;
-  Metalink2RequestGroup().generate(groups, "metalink4-groupbymetaurl.xml",
+  Metalink2RequestGroup().generate(groups,
+                                   A2_TEST_DIR"/metalink4-groupbymetaurl.xml",
                                    option_);
   CPPUNIT_ASSERT_EQUAL((size_t)3, groups.size());
 
@@ -211,7 +205,7 @@ void Metalink2RequestGroupTest::testGenerate_dosDirTraversal()
   std::vector<SharedHandle<RequestGroup> > groups;
   option_->put(PREF_DIR, "/tmp");
   Metalink2RequestGroup().generate
-    (groups, "metalink4-dosdirtraversal.xml", option_);
+    (groups, A2_TEST_DIR"/metalink4-dosdirtraversal.xml", option_);
   CPPUNIT_ASSERT_EQUAL((size_t)3, groups.size());
   SharedHandle<RequestGroup> rg = groups[0];
   SharedHandle<FileEntry> file = rg->getDownloadContext()->getFirstFileEntry();
